from application.core.parser_data import parse_data
from application.core.jobs.get_masked_or_deleted_db import delete_rows_from_db, mask_db
from application.core.parser_data import get_all_columns_by_query


def main():
    command_line_args, conn = parse_data()
    if conn is not None:
        if command_line_args.action == 'delete':
            if command_line_args.table and command_line_args.percentage:
                delete_rows_from_db(conn, command_line_args.table, command_line_args.percentage)
            else:
                print("No table or percentage data to be deleted has been entered")
        elif command_line_args.action == 'mask':
            if command_line_args.table:
                if not command_line_args.columns:
                    mask_db(conn, command_line_args.table, get_all_columns_by_query(command_line_args.table, conn))
                else:
                    mask_db(conn, command_line_args.table, command_line_args.columns)
            else:
                print("No table to be deleted has been entered")

        else:
            print("Invalid Action: ", command_line_args.action)
    else:
        print('Could not connect to database')


if __name__ == "__main__":
    main()



import re
from sqlalchemy import text


def delete_rows_from_db(connection, table_name, percentage):
    try:
        total_rows = connection.execute(text(f'SELECT COUNT(*) FROM {table_name}')).scalar()
        rows_to_delete = int(total_rows * (percentage / 100))
        delete_query = text(
            f'DELETE FROM {table_name} WHERE ctid IN (SELECT ctid FROM {table_name} ORDER BY RANDOM() LIMIT :limit)'
        ).bindparams(limit=rows_to_delete)
        connection.execute(delete_query)
        print(f"{percentage}% of data deleted from the {table_name} table.")
        connection.commit()
        connection.close()
    except Exception as e:
        print(f"Error: {e}")


def mask_data(connection, table_name, columns):
    try:
        for column in columns:
            max_length_query = text(f'SELECT MAX(LENGTH({column})) FROM {table_name}')
            max_length = connection.execute(max_length_query).scalar()
            mask_query = text(f'UPDATE {table_name} SET {column} = REPEAT(\'*\', {max_length})')
            connection.execute(mask_query)

        connection.commit()
        connection.close()
        print(f"Data in columns {', '.join(columns)} has been masked with '*' up to the maximum character length.")
    except Exception as e:
        print(f"Error: {e}")


def replace_email(match):
    email = match.group(0)
    if len(email) > 2:
        parts = email.split("@")
        username = parts[0]
        new_email = email[0] + '*' * (len(username) - 2) + username[-1] + '@' + parts[1]
        return new_email
    else:
        return email


def replace_phone_number(match):
    phone = match.group(0)
    return phone[:-4] + '****'


def replace_password(match):
    username = match.group(1)
    if len(match.group(2)) > 5:
        return f"{username}******"
    else:
        return f"{username}{match.group(2)}"


def replace_url(match):
    return 'https://*****.com/***'


def replace_credit_card_number(match):
    credit_card_number = match.group(0)
    return credit_card_number[:4] + '*' * (len(credit_card_number) - 8) + credit_card_number[
                                                                          len(credit_card_number) - 4:len(
                                                                              credit_card_number)]


def replace_birth_date(match):
    return re.sub(r'\d', '*', match.group())


def mask_db(connection, table_name, columns, chunk_size=10):
    try:
        tc_pattern = re.compile(r'\b(\d{4})[-.\s]?(\d{3})[-.\s]?(\d{4})\b')
        credit_card_pattern = re.compile(r'\b4[0-9]{12}(?:[0-9]{3})?\b')
        email_pattern = re.compile(r'\b[A-Za-z0-9._*%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b')
        phone_pattern = re.compile(r'\b(\d{3})[-.\s]?(\d{3})[-.\s]?(\d{4})\b')
        password_pattern = re.compile(r'(\b[A-Za-z0-9_-]+:)\s*\b([A-Za-z0-9_-]+)\b')
        url_pattern = re.compile(
            r'(https?://(?:www\.|(?!www))[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|www\.[a-zA-Z0-9]['
            r'a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|https?://(?:www\.|(?!www))[a-zA-Z0-9]+\.[^\s]{2,'
            r'}|www\.[a-zA-Z0-9]+\.[^\s]{2,})')
        birth_date_pattern = re.compile(r"(0[1-9]|[12][0-9]|3[01])[-/.](0[1-9]|1[012])[-/.](19|20)\d\d")

        total_rows = connection.execute(text(f"SELECT COUNT(*) FROM {table_name}")).scalar()

        for column in columns:

            for offset in range(0, total_rows, chunk_size):
                query = text(
                    f"SELECT id, {column} FROM {table_name} ORDER BY id LIMIT {chunk_size} OFFSET {offset}"
                )
                id_and_column_data = connection.execute(query).fetchall()

                for tuple_data in id_and_column_data:
                    row_id, old_value = tuple_data

                    regenerated_value = email_pattern.sub(replace_email, old_value)
                    regenerated_value = phone_pattern.sub(replace_phone_number, regenerated_value)
                    regenerated_value = password_pattern.sub(replace_password, regenerated_value)
                    regenerated_value = tc_pattern.sub(replace_phone_number, regenerated_value)
                    regenerated_value = credit_card_pattern.sub(replace_credit_card_number, regenerated_value)
                    regenerated_value = url_pattern.sub(replace_url, regenerated_value)
                    regenerated_value = birth_date_pattern.sub(replace_birth_date, regenerated_value)

                    update_query = text(
                        f"UPDATE {table_name} SET {column} = :regenerated_value WHERE id = :row_id"
                    )
                    connection.execute(update_query, {"regenerated_value": regenerated_value, "row_id": row_id})
                    connection.commit()
                print(f"Updated {chunk_size} rows in {table_name} for {column}, offset: {offset}")

    except Exception as e:
        print(f"Error: {e}")

    finally:
        connection.close()



import argparse
import os
from sqlalchemy import create_engine, text
from sqlalchemy.exc import SQLAlchemyError


def connection_db():
    try:
        db_url = os.environ.get('SQLALCHEMY_DATABASE_URI')
        engine = create_engine(db_url)
        conn = engine.connect()
        return conn
    except SQLAlchemyError as e:
        return None

def get_all_columns_by_query(table_name, connection):
    result = connection.execute(text(
        f"SELECT column_name FROM information_schema.columns WHERE table_name = '{table_name}' AND column_name <> 'id'"
    ))
    return [row[0] for row in result]


def parse_data():
    connection = connection_db()
    parser = argparse.ArgumentParser(description='Script for deleting or masking data from a database')
    parser.add_argument('--action', choices=['delete', 'mask'], help='Operation to be performed')
    parser.add_argument('--table', help='Name of the table from which data will be deleted or masked')
    parser.add_argument('--percentage', type=float, default=0, help='Percentage of data to be deleted (default: 0)')
    parser.add_argument('--columns', nargs='+', default=[], help='Columns to be masked')
    parser.add_argument('--run_mode', default=None)

    command_line_args = parser.parse_args()

    return command_line_args, connection



from flask import Flask


def get_sqlalchemy_db_url():
    try:
        app = Flask(__name__, template_folder="templates")
        config_file_location = None
        with app.app_context():
            if config_file_location:
                app.config.from_pyfile(config_file_location, silent=False)
            else:
                app.config.from_pyfile('../config/dev.cfg', silent=True)
                # app.config.from_pyfile('../config/prp.cfg', silent=True)

        sqlalchemy_database_url = app.config.get('SQLALCHEMY_DATABASE_URI')
        return sqlalchemy_database_url
    except Exception as e:
        return None



from setuptools import setup, find_packages

setup(
    name='masker',
    version='1.0.8',
    description='This is a data masking and file deletion tool designed to help you'
                ' protect sensitive information in your database and, if needed, delete'
                ' a percentage of files from a specified directory.',
    author='Aras',
    packages=find_packages(),
    install_requires=[
        'SQLAlchemy~=2.0.22',
        'flask~=2.2.5'
    ],
    package_data={
        'config': ['*.cfg'],
    },
    include_package_data=True,
    data_files=[('', ['run_masker/app.py', 'README.md', 'requirements.txt'])],
    entry_points={
        'console_scripts': [
            'masker=run_masker.app:main',
        ]
    }
)
